import {
  defineComponent,
  h,
  isVue2,
  markRaw,
  onMounted,
  onUnmounted,
  ref,
  watch,
  type PropType
} from "vue-demi";
import { ModContext } from "../types/ModContext";
import { InvokerItem } from "./InvokerItem";
import { modPageConfig } from "../modPageConfig";
import { hh } from "./DemiHelper";

let idCount = 0;
export type InvokerModName = string;

export default defineComponent({
  props: {
    height: {
      type: String,
      default: () => "auto",
    },
    scroll: {
      type: Boolean,
      default: () => false,
    },
    items: {
      type: Array,
      default: () => undefined as InvokerItem[] | undefined,
    },
    name: {
      type: String as PropType<InvokerModName> | undefined,
      default: () => undefined,
      required: false,
    }
  },
  emits: ["destroyed", "load"],
  setup(props, { slots, expose, emit }) {
    /** Invoker实例ID */
    const id = ++idCount;
    /** vue实例上下文，此处获取的是Receiver的vue组件实例 */
    let receiver: ModContext | null = null;
    const invokerKey = modPageConfig.Invoker.invokerKey;
    const url = ref<string | undefined>('');

    console.log(`[Mod-Invoker] 1. setting up invoker, id: ${id}, name: ${props.name}, invokerKey: ${invokerKey}`);

    window[invokerKey] ||= {};
    window[invokerKey][id] = {
      getRenderContext,
      initFinish,
      getReceiver() {
        return receiver;
      }
    };


    /** 主动更新 */
    function updateComponent() {
      // 判断子页面vue对象是否存在
      receiver?.Update(); // 强制渲染
      console.log("[Mod-Invoker] update component");
    }

    /** 当Receiver初始化完毕后由它触发 */
    function initFinish(context: ModContext) {
      receiver = context;
      console.log('[Mod-Invoker] 3.1. invoker init finished', receiver);
      emit('load');
    }

    function getRefs() {
      const refs = receiver?.refs;
      if (!refs) throw new Error("[Mod-Invoker] Receiver not found");
      return refs;
    }

    // vue3 渲染上下文
    function getRenderContext() {
      return markRaw(props.items);
    }

    
    // 用expose外部没有感知，用defineExpose在vue2会报错，没办法
    expose({
      getRefs,
      getRenderContext,
      initFinish,
      updateComponent
    });

    watch(
      () => props.items,
      () => {
        updateComponent();
      }
    );

    onMounted(async () => {
      url.value = await modPageConfig.Invoker.GetModUrl?.(props.name ?? '', id.toString(), invokerKey);
    })

    onUnmounted(() => {
      emit('destroyed');
    });

    return (createElem) => {
      const render = isVue2 ? createElem : h;
      if (slots.default) {
        return render('div', {}, slots.default());
      }

      return hh('iframe', {
        src: url.value,
        scrolling: props.scroll ? "yes" : "no",
        style: `border: none; width: 100%; height: ${props.height}; `
      });
    };
  },
});
